PROGRAM SOURCECOMPUTE

  implicit none
  include 'params.i'

  real*8, allocatable, dimension(:,:,:) :: vr
  allocate(vr(nx,ny,num_steps))

  call compute_source_spectrum(vr, num_steps, cadforcing)
  call writefits_3D(forcingfunc,vr,num_steps) 
  
  deallocate(vr)
END PROGRAM SOURCECOMPUTE


SUBROUTINE compute_source_spectrum(vr, number_steps, cadence)

  ! number_steps = NUMBER OF LEVELS OF FORCING FUNCTION SAMPLED AT 
  ! TEMPORAL RATE cadence ; cadence must be in seconds
  ! e.g. 8 hour long function at 1 minute cadence
  ! compute_source_spectrum(480, 60.)

   use Ziggurat
   implicit none
   include 'fftw3.f'
   include 'params.i'
   integer *8 plan3d
   integer nelements,sendtag,recvtag,tempindex,rprev,sprev,rem,maxwave
   integer i,j,k,max_nu,min_nu,l,m, number_steps, ierr,p
   real*8 frequency_spacing, max_frequency, min_frequency,rdm1,rdm2,excite(5)
   real*8 normal,nu_mhz, spacereq, cadence, kx(1:nx),ky(1:ny),gk,dx,dy,pi
   real*8 mean, std,spectrum(number_steps),om(number_steps/2+1),vr(nx,ny,number_steps)
   complex*16 frequency_series(number_steps/2+1), constant, eye
   complex*16 tempin(ny), tempout(ny), tempcomp(nx), tempreal(nx)
   real*8 decay_kx(nx), decay_ky(ny),time_series(number_steps),me,lambdamin,lmax
   real*8, allocatable, dimension(:,:,:) :: decay_coeffs, temp3d
   complex*16, allocatable, dimension(:,:,:) :: coef_forcing, coeffs
   
   allocate(coef_forcing(number_steps/2+1,nx,ny),decay_coeffs(nx,ny,number_steps/2+1), temp3d(number_steps,nx,ny) )
  
    pi = ACOS(-1.0D0)
   eye = (0.0d0, 1.0d0)
   i = 42 
   CALL zigset( i )

   dx = xlength/(nx*1.)
   dy = dx
   call dfftw_plan_dft_c2r_3d(plan3d,number_steps,nx,ny,coef_forcing,temp3d,FFTW_ESTIMATE)


   max_frequency = 0.0073
   min_frequency = 0.0001
   mean = 0.00
   std  = 0.001
   frequency_spacing = 1.0/(DBLE(number_steps)*cadence) ! Cadence in seconds
   max_nu = floor(max_frequency/frequency_spacing) + 1
   min_nu = floor(min_frequency/frequency_spacing)
   tempindex = floor(0.005/frequency_spacing)
   constant = (0.5,0.5)
   spectrum(:) = 0.0

   excite(1) = -26.319250 
   excite(2) = 0.050174650
   excite(3) = -3.5238502e-05
   excite(4) = 1.0667517e-08
   excite(5) = -1.0979302e-12
 
   frequency_series = (0.0,0.0)
 
   normal = 0.0
   spectrum(:) = 0.0
  ! SPECTRUM IS THE FREQUENCY SPECTRAL DISTRIBUTION OF THE FORCING FUNCTION. CHANGE AS YOU WISH
  ! nu_mhz is the FREQUENCY IN mHz
   do i = min_nu,max_nu!tempindex !max_nu
    nu_mhz = frequency_spacing*(i-1.0)*10.0**(3.0)

    spectrum(i) = exp(-(nu_mhz - 3.)**2.0/7.5) * 1.0/(1.0 + exp((nu_mhz-7.5)*10.0)) !/(1.0 + exp(-(nu_mhz-1.4)*8.0))
    if (spectrum(i) .GT. normal) normal = spectrum(i)
   enddo

   spectrum = spectrum/normal

   ky(:) = 0.0
   kx(:) = 0.0

   decay_ky(1) = 0.0 
   do i=2,ny/3+1
    decay_ky(i) = 1.0/(1.0 + exp((i-nx/3.)*3.))!*1.0/(1.0+exp(2.5*(2.0-i))) ! 
    decay_ky(ny-i+2) = decay_ky(i)
   enddo
   ! The nyquist is located at ny/2+1 because the first index corresponds to zero.
   !decay_ky(ny/2+1) = 0.0 

   ! To ensure zero mean in x
   decay_kx(1) = 0.0
   do i=2,nx/3+1
    decay_kx(i) =  1.0/(1.0 + exp((i-nx/3.)*3.))!1.0/(1.0+exp(2.5*(2.0-i))) !1.0/(1.0 + exp((i-maxwave)*0.5))!* 
    decay_kx(nx-i+2) = decay_kx(i)
   enddo
   !decay_kx(nx/2+1) = 0.0
   decay_kx = decay_ky

   ky(1) = 0.0 
   do j=2,ny/2
    ky(j) = 2.0*pi*(j-1.0)/(ylength)
    ky(ny-j+2) = -ky(j)
   enddo
   ! Nyquist in the y direction
   ky(ny/2+1) = -2.0*pi*0.5/dy

   kx(1) = 0.0
   do j=2,nx/2
    kx(j) = 2.0*pi*(j-1.0)/(xlength)
    kx(ny-j+2) = -kx(j)
   enddo
   ! Nyquist in the x direction
   kx(nx/2+1) = -2.0*pi*0.5/dx

   om(1) = 0.0
   do k = 2,number_steps/2
    om(k) = frequency_spacing*(k-1.0)*2.0*pi
   enddo
   ! The frequency nyquist
   om(number_steps/2+1) = -0.5*frequency_spacing*number_steps*2.0*pi


   ! One way phase speed filter
   do k=1,number_steps/2+1
    do j=1,ny
     do i=1,nx
      gk = 2.50517592953322*10.0**4.0*(kx(i)**2.0 +ky(j)**2.0)**0.5
!      lamb = abs(2*pi*2.50517592953322*10.0**(-4.0)/gk - 32.)
      decay_coeffs(i,j,k) = decay_kx(i)*&!/(1.0 + exp((gk*0.8 - om(k)**2.0)/0.0000006))* &
				decay_ky(j)*spectrum(k)!*(1. + 0.2/(1. + exp(lamb)))
     enddo
    enddo
   enddo
       
   coef_forcing(:,:,:) =0.0

   do k=1,ny
    do j=1,nx
     frequency_series(:) = (0.0, 0.0)
     do i = min_nu,max_nu
      rdm1 = rnor()
      rdm2 = rnor()
      coef_forcing(i,j,k) = 10.0**(-8.0)  *(rdm1+ eye*rdm2) * decay_coeffs(j,k,i)
     enddo
    enddo
   enddo
 
   call dfftw_execute(plan3d)

   do k=1,number_steps
    me = 0.0
    do j=1,ny
     do i=1,nx
      vr(i,j,k) = temp3d(k,i,j)!*(1.0-spongex(i))*(1.0 -spongey(j))
      me = me + vr(i,j,k)
     enddo
    enddo
    vr(:,:,k) = vr(:,:,k) - me/(nx*ny*1.)
  enddo

  call dfftw_destroy_plan(plan3d)
  deallocate(coef_forcing,decay_coeffs,temp3d)


end SUBROUTINE compute_source_spectrum

!================================================================================================ 

      SUBROUTINE printerror(status)

      ! See cookbook.f on FITSIO for details

      integer status
      character errtext*30,errmessage*80

      if (status .le. 0)return

      call ftgerr(status,errtext)
      print *,'FITSIO Error Status =',status,': ',errtext

      call ftgmsg(errmessage)
      do while (errmessage .ne. ' ')
          print *,errmessage
          call ftgmsg(errmessage)
      end do

      end SUBROUTINE printerror

!================================================================================
      SUBROUTINE deletefile(filename,status)

      integer status,unit,blocksize
      character*(*) filename

      if (status .gt. 0)return
      call ftgiou(unit,status)
      call ftopen(unit,filename,1,blocksize,status)
      

      if (status .eq. 0)then
          call ftdelt(unit,status)
      endif
      if (status .eq. 103)then
          status=0
          call ftcmsg
      endif 
      if ((status .NE. 0) .and. (status .ne. 103)) then 

          status=0
          call ftcmsg
          call ftdelt(unit,status)
      end if

      call ftfiou(unit, status)

      end SUBROUTINE deletefile

!===============================================================================

     SUBROUTINE writefits_3D(filename,dump_array,dim3)

      implicit none 
      include 'params.i'
      integer blocksize,bitpix,naxes(3),unit1,k,disp, tag, dim3
      integer status1,group,fpixel
      integer temptype, sendtyp, nelements
      real*8 dump_array(nx,ny,dim3)
      character*(*) filename
      logical simple,extend

         status1 = 0
         call ftgiou(unit1,status1)
         blocksize=1

         call ftinit(unit1,filename,blocksize,status1)
         simple=.true.
         bitpix=-64
         naxes(1)=nx
         naxes(2)=ny
         naxes(3)=dim3
         nelements=naxes(1)*naxes(2)*naxes(3)
         extend=.false.
         group=1
         fpixel=1
         
         call ftphpr(unit1,simple,bitpix,3,naxes,0,1,extend,status1)
         !call ftpkyj(unit1,'TIME',time,'Also, init = time+1', status1)
         call ftpprd(unit1,group,fpixel,nelements,dump_array,status1)
         call ftclos(unit1, status1)
         call ftfiou(unit1, status1)


     end SUBROUTINE writefits_3D



